package org.repro;

import com.github.database.rider.cdi.api.DBRider;
import com.github.database.rider.core.api.dataset.DataSet;
import io.quarkus.test.junit.QuarkusTest;
import org.junit.jupiter.api.Test;
import org.repro.model.A;
import org.repro.model.B;
import org.repro.model.RX;
import org.repro.model.UX;

import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.transaction.Transactional;
import java.util.List;

import static org.assertj.core.api.AssertionsForClassTypes.assertThat;

@QuarkusTest
@DBRider
public class ReproTest {

    @Inject
    EntityManager entityManager;

    @Test
    @DataSet(value = "org/repro/ReproTest.yml")
    @Transactional // this annotation is required to reproduce the problem
    public void reproducer_SanityCheck() {
        // we have one A, with one B, with one C
        List<A> aList = entityManager.createQuery("select a from A a").getResultList();
        assertThat(aList.size()).isEqualTo(1);

        A a = aList.get(0);
        B b = a.getB();
        assertThat(b).isNotNull();

        int numBs = b.getCList().size();
        assertThat(numBs).isEqualTo(1);
    }

    @Test
    @DataSet(value = "org/repro/ReproTest.yml")
    @Transactional // this annotation is required to reproduce the problem
    public void reproducer_Case1() {
        List<A> aList = entityManager.createQuery("select a from A a").getResultList();
        assertThat(aList.size()).isEqualTo(1);

        A a = aList.get(0);
        B b = a.getB();
        assertThat(b).isNotNull();

        // after we do a listAll() on UX...
        List<UX> list = entityManager.createQuery("select ux from UX ux").getResultList();

        // ... it seems as if B does not have any C
        int numCs = b.getCList().size();
        System.out.println("numBs: " + numCs);
        assertThat(numCs).isEqualTo(1);
    }

    @Test
    @DataSet(value = "org/repro/ReproTest.yml")
    @Transactional // this annotation is required to reproduce the problem
    public void reproducer_Case2() {
        List<A> aList = entityManager.createQuery("select a from A a").getResultList();
        assertThat(aList.size()).isEqualTo(1);

        A a = aList.get(0);
        B b = a.getB();
        assertThat(b).isNotNull();

        // after we do a listAll() on UX...
        List<RX> list = entityManager.createQuery("select rx from RX rx").getResultList();

        // ... it seems as if B does not have any C
        int numBs = b.getCList().size();
        System.out.println("numBs: " + numBs);
        assertThat(numBs).isEqualTo(1); // because of the problem we get 0 instead of 1
    }
}
